#!/sw/lib/apache/internal/cgi/apache/nph-eperl
<html>
<head>
<title>demo-01: CGI variable usage</title>
</head>
<body>
<h1>demo-01: CGI variable usage</h1>

Welcome user from <? print $ENV{'REMOTE_HOST'}; !>, nice to meet you!

</body>
</html>
