<html>
<head>
<title>demo-04: A Syntax Error</title>
</head>
<body>
<h1>demo-04: A Syntax Error</h1>

Here come the numbers from 0 to 100: 
<? for($i := 0; $i <= 100; $i++) { print $i . " "; } !>
and then again pure text.

</body>
</html>
