<html>
<head>
<title>demo-05: A STDERR output.</title>
</head>
<body>
<h1>demo-05: A STDERR output.</h1>

Here come the numbers from 0 to 100: 
<? for($i = 0; $i <= 100; $i++) { print $i . " "; } 
   print STDERR "This is data on STDERR"; !>
and then again pure text.

</body>
</html>
