/* LICENSE.c -- automatically generated by asc2c */

char *ePerl_LICENSE = \
"\n"\
"  e P e r l   L I C E N S E   A G R E E M E N T\n"\
"\n"\
"  Copyright (c) 1996,1997 Ralf S. Engelschall, All rights reserved.\n"\
"  \n"\
"  Redistribution and use in source and binary forms, with or without\n"\
"  modification, are permitted provided that the following conditions\n"\
"  are met:\n"\
"  \n"\
"  1. Redistributions of source code must retain the above copyright\n"\
"     notice, this list of conditions and the following disclaimer. \n"\
"  \n"\
"  2. Redistributions in binary form must reproduce the above copyright\n"\
"     notice, this list of conditions and the following disclaimer in\n"\
"     the documentation and/or other materials provided with the\n"\
"     distribution.\n"\
"  \n"\
"  3. All advertising materials mentioning features or use of this\n"\
"     software must display the following acknowledgment:\n"\
"         \"This product includes software developed by \n"\
"          Ralf S. Engelschall <rse@engelschall.com>.\"\n"\
"  \n"\
"  4. Redistributions of any form whatsoever must retain the following\n"\
"     acknowledgment:\n"\
"         \"This product includes software developed by \n"\
"          Ralf S. Engelschall <rse@engelschall.com>.\"\n"\
"  \n"\
"  5. The names \"ePerl\" and \"Embedded Perl 5 Language\" must not be used to\n"\
"     endorse or promote products derived from this software without\n"\
"     prior written permission.\n"\
"  \n"\
"  THIS SOFTWARE IS PROVIDED BY RALF S. ENGELSCHALL ``AS IS'' AND ANY\n"\
"  EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE\n"\
"  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE\n"\
"  ARE DISCLAIMED.  IN NO EVENT SHALL RALF S. ENGELSCHALL OR HIS CONTRIBUTORS\n"\
"  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR\n"\
"  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF\n"\
"  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS\n"\
"  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN\n"\
"  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)\n"\
"  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE\n"\
"  POSSIBILITY OF SUCH DAMAGE.\n"\
"\n"\
;

/*EOF*/
