Content-type: text/html
X-TestHeader: FooBarQuux

<html>
<head>
<title>demo-07: Custom HTTP headers</title>
</head>
<body>
<h1>demo-07: Custom HTTP headers</h1>

Welcome user from <? print $ENV{'REMOTE_HOST'}; !>, nice to meet you!

</body>
</html>
