/* README.c -- automatically generated by asc2c */

char *ePerl_README = \
"        ____           _ \n"\
"    ___|  _ \\ ___ _ __| |\n"\
"   / _ \\ |_) / _ \\ '__| |\n"\
"  |  __/  __/  __/ |  | |\n"\
"   \\___|_|   \\___|_|  |_|\n"\
"                         \n"\
"  ePerl -- Embedded Perl 5 Language\n"\
"\n"\
"  Version 2.1b4 (03-04-1997)\n"\
"\n"\
"  ePerl interprets an ASCII file bristled with Perl 5 program statements by\n"\
"  evaluating the Perl 5 code while passing through the plain ASCII data. It\n"\
"  can operate both as a standard Unix filter for file generation and as a\n"\
"  Webserver scripting language for dynamic HTML page programming. \n"\
"\n"\
"  The documentation and latest release can be found on\n"\
"  http://www.engelschall.com/sw/eperl/\n"\
"\n"\
"  Copyright (c) 1996,1997 Ralf S. Engelschall, All rights reserved.\n"\
"\n"\
"  Look at the file LICENSE or run ``eperl -l'' for detailed license \n"\
"  information about this product.\n"\
"\n"\
"\n"\
;

/*EOF*/
